@@ -24,6 +24,8 @@
 # include <config.h>
 #endif
 
+#include <libxml/tree.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_system_characteristics_impl.h"
@@ -188,12 +190,12 @@ void oval_record_field_free(struct oval_record_field *rf)
 
 void oval_record_field_set_name(struct oval_record_field *rf, char *name)
 {
-	rf->name = name;
+	rf->name = oscap_strdup(name);
 }
 
 void oval_record_field_set_value(struct oval_record_field *rf, char *value)
 {
-	rf->value = value;
+	rf->value = oscap_strdup(value);
 }
 
 void oval_record_field_set_datatype(struct oval_record_field *rf, oval_datatype_t dt)
@@ -354,6 +356,7 @@ int oval_record_field_parse_tag(xmlTextReaderPtr reader, struct oval_parser_cont
 	mask = oval_parser_boolean_attribute(reader, "mask", 0);
 
 	oval_record_field_set_name(rf, name);
+	xmlFree(name);
 	oval_record_field_set_datatype(rf, datatype);
 	oval_record_field_set_mask(rf, mask);
 
@@ -406,7 +409,7 @@ int oval_record_field_parse_tag(xmlTextReaderPtr reader, struct oval_parser_cont
 	return ret;
 }
 
-xmlNode *oval_record_field_to_dom(struct oval_record_field *rf, bool parent_mask, xmlDoc *doc, xmlNode *parent)
+xmlNode *oval_record_field_to_dom(struct oval_record_field *rf, bool parent_mask, xmlDoc *doc, xmlNode *parent, xmlNs *namespace)
 {
 	char *name, *value;
 	bool rf_mask, masked;
@@ -433,7 +436,7 @@ xmlNode *oval_record_field_to_dom(struct oval_record_field *rf, bool parent_mask
 		masked = false;
 	}
 
-	node = xmlNewTextChild(parent, NULL, BAD_CAST "field", BAD_CAST value);
+	node = xmlNewTextChild(parent, namespace, BAD_CAST "field", BAD_CAST value);
 	xmlNewProp(node, BAD_CAST "name", BAD_CAST name);
 	datatype = oval_record_field_get_datatype(rf);
 	if (datatype != OVAL_DATATYPE_STRING)
