@@ -41,6 +41,7 @@
 #include "DS/ds_common.h"
 #include "XCCDF_POLICY/public/xccdf_policy.h"
 #include "XCCDF_POLICY/xccdf_policy_priv.h"
+#include "XCCDF_POLICY/xccdf_policy_model_priv.h"
 #include "item.h"
 #include "public/xccdf_session.h"
 #include "XCCDF_POLICY/public/check_engine_plugin.h"
@@ -882,8 +883,9 @@ int xccdf_session_load_tailoring(struct xccdf_session *session)
 		free(xccdf_tailoring_version);
 	}
 
+	struct xccdf_benchmark *benchmark = xccdf_policy_model_get_benchmark(session->xccdf.policy_model);
 	struct xccdf_tailoring *tailoring = xccdf_tailoring_import(tailoring_path,
-		xccdf_policy_model_get_benchmark(session->xccdf.policy_model));
+		benchmark);
 
 	free(tailoring_path);
 
@@ -986,6 +988,11 @@ int xccdf_session_export_xccdf(struct xccdf_session *session)
 
 	/* Export results */
 	if (session->export.xccdf_file != NULL) {
+		if (session->xccdf.result == NULL) {
+			// Attempt to export session before evaluation
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "No XCCDF results to export.");
+			return 1;
+		}
 		xccdf_benchmark_add_result(xccdf_policy_model_get_benchmark(session->xccdf.policy_model),
 				xccdf_result_clone(session->xccdf.result));
 		xccdf_benchmark_export(xccdf_policy_model_get_benchmark(session->xccdf.policy_model), session->export.xccdf_file);
@@ -1127,6 +1134,7 @@ static char *_xccdf_session_get_unique_oval_result_filename(struct xccdf_session
 			break;
 
 		free(name);
+		name = NULL;
 		++suffix;
 	}
 
@@ -1313,8 +1321,11 @@ int xccdf_session_export_arf(struct xccdf_session *session)
 			ds_sds_compose_from_xccdf(session->filename, sds_path);
 		}
 
-		ds_rds_create(sds_path, session->export.xccdf_file, (const char**)(session->oval.result_files), session->export.arf_file);
+		int res = ds_rds_create(sds_path, session->export.xccdf_file, (const char**)(session->oval.result_files), session->export.arf_file);
 		free(sds_path);
+		if (res != 0) {
+			return res;
+		}
 
 		if (session->full_validation) {
 			if (oscap_validate_document(session->export.arf_file, OSCAP_DOCUMENT_ARF, "1.1", _reporter, NULL)) {
@@ -1393,7 +1404,7 @@ int xccdf_session_remediate(struct xccdf_session *session)
 		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Can't perform remediation in offline mode: not implemented");
 		return 1;
 	}
-	xccdf_policy_model_unregister_callbacks(session->xccdf.policy_model, oval_sysname);
+	xccdf_policy_model_unregister_engines(session->xccdf.policy_model, oval_sysname);
 	if ((res = xccdf_session_load_oval(session)) != 0)
 		return res;
 	xccdf_result_fill_sysinfo(session->xccdf.result);
