@@ -1,7 +1,7 @@
 /*
  * xccdf_policy.c
  *
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009--2014 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -32,6 +32,9 @@
 #include <libgen.h>
 
 #include "xccdf_policy_priv.h"
+#include "xccdf_policy_model_priv.h"
+#include "xccdf_policy_engine_priv.h"
+#include "reporter_priv.h"
 #include "public/xccdf_policy.h"
 #include "public/xccdf_benchmark.h"
 #include "public/oscap_text.h"
@@ -45,32 +48,7 @@
 #include "common/_error.h"
 #include "common/debug_priv.h"
 #include "common/assume.h"
-
-/**
- * Typedef of callback structure with system identificator, callback function and usr data (optional) 
- * On evaluation action will be selected checking system and appropriate callback registred by tool
- * for that system.
- */
-typedef struct callback_t {
-
-    char * system;                              ///< Identificator of checking engine
-	xccdf_policy_engine_eval_fn callback;	///< format of callback function
-    void * usr;                                 ///< User data structure
-    xccdf_policy_engine_query_fn query_fn;      ///< query callback function
-
-} callback;
-
-/**
- * Typedef of callback structure with callback function and usr data (optional)
- * After rule evaluation action will be called the callback with user data.
- */
-typedef struct callback_out_t {
-
-    char * system;                              ///< Identificator of checking engine (output engine)
-    int (*callback)(void*,void*);             	///< policy report callback {output,start}
-    void * usr;                                 ///< User data structure
-
-} callback_out;
+#include "common/text_priv.h"
 
 /**
  * XCCDF policy model structure contains xccdf_benchmark as reference
@@ -82,10 +60,13 @@ struct xccdf_policy_model {
         struct xccdf_benchmark  * benchmark;    ///< Benchmark element (root element of XML file)
 	struct xccdf_tailoring * tailoring;     ///< Tailoring element
 	struct oscap_list       * policies;     ///< List of xccdf_policy structures
-        struct oscap_list       * callbacks;    ///< Callbacks for checking engines (see callback_t)
-        struct oscap_list       * cpe_dicts; ///< All CPE dictionaries except the one embedded in XCCDF
+	struct oscap_list       * callbacks;    ///< Callbacks for output callbacks (see callback_out_t)
+	struct oscap_list       * engines;      ///< Callbacks for checking engines (see xccdf_policy_engine)
+
+	struct oscap_list       * cpe_dicts; ///< All CPE dictionaries except the one embedded in XCCDF
 	struct oscap_list       * cpe_lang_models; ///< All CPE lang models except the one embedded in XCCDF
-        struct oscap_htable     * cpe_oval_sessions; ///< Caches CPE OVAL check results
+	struct oscap_htable     * cpe_oval_sessions; ///< Caches CPE OVAL check results
+	struct oscap_htable     * cpe_applicable_platforms;
 };
 /* Macros to generate iterators, getters and setters */
 OSCAP_GETTER(struct xccdf_benchmark *, xccdf_policy_model, benchmark)
@@ -192,16 +173,6 @@ static bool xccdf_policy_filter_selected(void *item, void *policy)
 		(xccdf_select_get_selected((struct xccdf_select *) item)));
 }
 
-/**
- * Filter function returning true if given callback is for the given checking engine,
- * false otherwise.
- */
-static bool
-_xccdf_policy_filter_callback(callback *cb, const char *sysname)
-{
-	return oscap_strcmp(cb->system, sysname) == 0;
-}
-
 /**
  * Get callbacks that match sysname. Call this function to get iterator over list of callbacks
  * that have the same system name
@@ -209,7 +180,40 @@ _xccdf_policy_filter_callback(callback *cb, const char *sysname)
 static struct oscap_iterator *
 _xccdf_policy_get_callbacks_by_sysname(struct xccdf_policy * policy, const char * sysname)
 {
-	return oscap_iterator_new_filter( policy->model->callbacks, (oscap_filter_func) _xccdf_policy_filter_callback, (void *) sysname);
+	return oscap_iterator_new_filter( policy->model->callbacks, (oscap_filter_func) xccdf_policy_engine_filter, (void *) sysname);
+}
+
+/**
+ * Get the checking engines that match the sysname.
+ */
+static struct oscap_iterator *_xccdf_policy_get_engines_by_sysname(struct xccdf_policy *policy, const char *sysname)
+{
+	return oscap_iterator_new_filter(policy->model->engines, (oscap_filter_func) xccdf_policy_engine_filter, (void *) sysname);
+}
+
+char *xccdf_policy_get_readable_item_title(struct xccdf_policy *policy, struct xccdf_item *item, const char *preferred_lang)
+{
+	struct oscap_text_iterator *title_it = xccdf_item_get_title(item);
+	char *unresolved = oscap_textlist_get_preferred_plaintext(title_it, preferred_lang);
+	oscap_text_iterator_free(title_it);
+	char *resolved = xccdf_policy_substitute(unresolved, policy);
+	oscap_free(unresolved);
+	return resolved;
+}
+
+char *xccdf_policy_get_readable_item_description(struct xccdf_policy *policy, struct xccdf_item *item, const char *preferred_lang)
+{
+	/* Get description in prefered language */
+	struct oscap_text_iterator *description_it = xccdf_item_get_description(item);
+	struct oscap_text *unresolved_text = oscap_textlist_get_preferred_text(description_it, preferred_lang);
+	oscap_text_iterator_free(description_it);
+	const char *unresolved = oscap_text_get_text(unresolved_text);
+	/* Resolve <xccdf:sub> elements */
+	const char *resolved = xccdf_policy_substitute(unresolved, policy);
+	/* Get a rid of xhtml elements */
+	char *plaintext = _xhtml_to_plaintext(resolved);
+	oscap_free(resolved);
+	return plaintext;
 }
 
 /**
@@ -416,23 +420,13 @@ static void xccdf_policy_resolve_item(struct xccdf_policy * policy, struct xccdf
  */
 static int
 xccdf_policy_evaluate_cb(struct xccdf_policy * policy, const char * sysname, const char * content, const char * href,
-        const char * rule_id, struct oscap_list * bindings, struct xccdf_check_import_iterator * check_import_it)
+		struct oscap_list * bindings, struct xccdf_check_import_iterator * check_import_it)
 {
     xccdf_test_result_type_t retval = XCCDF_RESULT_NOT_CHECKED;
-    struct oscap_iterator * cb_it = _xccdf_policy_get_callbacks_by_sysname(policy, sysname);
+    struct oscap_iterator * cb_it = _xccdf_policy_get_engines_by_sysname(policy, sysname);
     while (oscap_iterator_has_more(cb_it)) {
-        callback * cb = (callback *) oscap_iterator_next(cb_it);
-        if (cb == NULL) { /* No callback found - checking system not registered */
-            oscap_seterr(OSCAP_EFAMILY_XCCDF, "Unknown callback for given checking system. Set callback first");
-            oscap_iterator_free(cb_it);
-            return XCCDF_RESULT_NOT_CHECKED;
-        }
-
-        struct xccdf_value_binding_iterator * binding_it = (struct xccdf_value_binding_iterator *) oscap_iterator_new(bindings);
-
-        retval = cb->callback(policy, rule_id, content, href, binding_it, check_import_it, cb->usr);
-
-        if (binding_it != NULL) xccdf_value_binding_iterator_free(binding_it);
+        struct xccdf_policy_engine *engine = (struct xccdf_policy_engine *) oscap_iterator_next(cb_it);
+	retval = xccdf_policy_engine_eval(engine, policy, content, href, bindings, check_import_it);
         if (retval != XCCDF_RESULT_NOT_CHECKED) break;
     }
     oscap_iterator_free(cb_it);
@@ -448,15 +442,13 @@ xccdf_policy_evaluate_cb(struct xccdf_policy * policy, const char * sysname, con
 static struct oscap_stringlist *
 _xccdf_policy_get_namesfor_href(struct xccdf_policy *policy, const char *sysname, const char *href)
 {
-	struct oscap_iterator *cb_it = _xccdf_policy_get_callbacks_by_sysname(policy, sysname);
+	struct oscap_iterator *cb_it = _xccdf_policy_get_engines_by_sysname(policy, sysname);
 	struct oscap_stringlist *result = NULL;
 	while (oscap_iterator_has_more(cb_it) && result == NULL) {
-		callback *cb = (callback *) oscap_iterator_next(cb_it);
-		if (cb == NULL)
+		struct xccdf_policy_engine *engine = (struct xccdf_policy_engine *) oscap_iterator_next(cb_it);
+		if (engine == NULL)
 			break;
-		if (cb->query_fn == NULL)
-			continue;
-		result = (struct oscap_stringlist *) cb->query_fn(cb->usr, POLICY_ENGINE_QUERY_NAMES_FOR_HREF, (void *)href);
+		result = xccdf_policy_engine_query(engine, POLICY_ENGINE_QUERY_NAMES_FOR_HREF, (void *) href);
 	}
 	oscap_iterator_free(cb_it);
 	return result;
@@ -467,10 +459,9 @@ int xccdf_policy_report_cb(struct xccdf_policy *policy, const char *sysname, voi
     int retval = 0;
     struct oscap_iterator * cb_it = _xccdf_policy_get_callbacks_by_sysname(policy, sysname);
     while (oscap_iterator_has_more(cb_it)) {
-        callback_out * cb = (callback_out *) oscap_iterator_next(cb_it);
+        struct reporter *cb = (struct reporter *) oscap_iterator_next(cb_it);
 
-        /* Report */
-	retval = cb->callback(rule, cb->usr);
+	retval = reporter_send_simple(cb, rule);
 
         /* We still want to stop evaluation if user cancel it
          * TODO: We should have a way to stop evaluation of current item
@@ -519,7 +510,7 @@ static struct oscap_list * xccdf_policy_check_get_value_bindings(struct xccdf_po
             if (r_value != NULL) {
                 selector = xccdf_refine_value_get_selector(r_value);
                 /* This refine value changes the value content */
-                if (xccdf_refine_value_get_oper(r_value) > 0) {
+                if ((int)xccdf_refine_value_get_oper(r_value) > 0) {
                     binding->operator = xccdf_refine_value_get_oper(r_value);
                 } else binding->operator = xccdf_value_get_oper(value);
 
@@ -589,7 +580,7 @@ int xccdf_policy_check_evaluate(struct xccdf_policy * policy, struct xccdf_check
                 href = xccdf_check_content_ref_get_href(content);
 
                 struct xccdf_check_import_iterator * check_import_it = xccdf_check_get_imports(check);
-                ret = xccdf_policy_evaluate_cb(policy, system_name, content_name, href, NULL, bindings, check_import_it);
+                ret = xccdf_policy_evaluate_cb(policy, system_name, content_name, href, bindings, check_import_it);
                 // the evaluation has filled check imports at this point, we can simply free the iterator
                 xccdf_check_import_iterator_free(check_import_it);
 
@@ -612,7 +603,7 @@ int xccdf_policy_check_evaluate(struct xccdf_policy * policy, struct xccdf_check
 static inline bool
 _xccdf_policy_is_engine_registered(struct xccdf_policy *policy, char *sysname)
 {
-	return oscap_list_contains(policy->model->callbacks, (void *) sysname, (oscap_cmp_func) _xccdf_policy_filter_callback);
+	return oscap_list_contains(policy->model->engines, (void *) sysname, (oscap_cmp_func) xccdf_policy_engine_filter);
 }
 
 static struct xccdf_check *
@@ -722,8 +713,24 @@ static struct xccdf_rule_result * _xccdf_rule_result_new_from_rule(const struct
 		xccdf_rule_result_add_ident(rule_ritem, xccdf_ident_clone(ident));
 	}
 	xccdf_ident_iterator_free(ident_it);
-	if (check != NULL)
+	if (check != NULL) {
 		xccdf_rule_result_add_check(rule_ritem, check);
+	} else {
+		/* Workaround: No applicable or candidate check was found. However,
+		 * the ARF schematron requires us to include at least one check.
+		 *
+		 * For more info see thread on xccdf-dev mailing list named:
+		 *     [Xccdf-dev] xccdf:rule-result element properties
+		 */
+		struct xccdf_check_iterator *check_it = xccdf_rule_get_checks(rule);
+		while (xccdf_check_iterator_has_more(check_it)) {
+			struct xccdf_check *orig_check = xccdf_check_iterator_next(check_it);
+			struct xccdf_check *new_check = xccdf_check_clone(orig_check);
+			xccdf_rule_result_add_check(rule_ritem, new_check);
+		}
+		xccdf_check_iterator_free(check_it);
+
+	}
 
 	if (message != NULL) {
 		struct xccdf_message *msg = xccdf_message_new();
@@ -816,6 +823,11 @@ static bool _xccdf_policy_cpe_check_cb(const char* sys, const char* href, const
 		}
 
 		session = oval_agent_new_session(oval_model, prefixed_href);
+		if (session == NULL) {
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Cannot create OVAL session for '%s' for CPE applicability checking", prefixed_href);
+			oscap_free(prefixed_href);
+			return false;
+		}
 		oscap_htable_add(model->cpe_oval_sessions, prefixed_href, session);
 	}
 	oscap_free(prefixed_href);
@@ -885,7 +897,10 @@ static bool xccdf_policy_model_platforms_are_applicable_dict(struct xccdf_policy
 		if (applicable)
 		{
 			ret = true;
-			break;
+
+			if (oscap_htable_get(model->cpe_applicable_platforms, platform) == NULL) {
+				oscap_htable_add(model->cpe_applicable_platforms, platform, 0);
+			}
 		}
 	}
 	oscap_string_iterator_reset(platforms);
@@ -923,7 +938,10 @@ static bool xccdf_policy_model_platforms_are_applicable_lang_model(struct xccdf_
 		if (applicable)
 		{
 			ret = true;
-			break;
+
+			if (oscap_htable_get(model->cpe_applicable_platforms, platform) == NULL) {
+				oscap_htable_add(model->cpe_applicable_platforms, platform, 0);
+			}
 		}
 	}
 	oscap_string_iterator_reset(platforms);
@@ -938,6 +956,7 @@ bool xccdf_policy_model_platforms_are_applicable(struct xccdf_policy_model *mode
 	if (!oscap_string_iterator_has_more(platforms))
 		return true;
 
+	bool ret = false;
 	// We do not check whether the platform entries are valid platform refs
 	// or CPE names. We let the policy_model methods do that instead.
 	// Therefore we check all 4 (!) places where a platform may match.
@@ -947,35 +966,32 @@ bool xccdf_policy_model_platforms_are_applicable(struct xccdf_policy_model *mode
 	struct cpe_lang_model *embedded_lang_model = xccdf_benchmark_get_cpe_lang_model(benchmark);
 	if (embedded_lang_model != NULL) {
 		if (xccdf_policy_model_platforms_are_applicable_lang_model(model, embedded_lang_model, platforms))
-			return true;
+			ret = true;
 	}
 
 	struct oscap_iterator *lang_models = oscap_iterator_new(model->cpe_lang_models);
 	while (oscap_iterator_has_more(lang_models)) {
 		struct cpe_lang_model *lang_model = (struct cpe_lang_model *) oscap_iterator_next(lang_models);
-		if (xccdf_policy_model_platforms_are_applicable_lang_model(model, lang_model, platforms)) {
-			oscap_iterator_free(lang_models);
-			return true;
-		}
+		if (xccdf_policy_model_platforms_are_applicable_lang_model(model, lang_model, platforms))
+			ret = true;
 	}
 	oscap_iterator_free(lang_models);
 
 	struct cpe_dict_model *embedded_dict = xccdf_benchmark_get_cpe_list(benchmark);
 	if (embedded_dict != NULL) {
 		if (xccdf_policy_model_platforms_are_applicable_dict(model, embedded_dict, platforms))
-			return true;
+			ret = true;
 	}
 
 	struct oscap_iterator *dicts = oscap_iterator_new(model->cpe_dicts);
 	while (oscap_iterator_has_more(dicts)) {
 		struct cpe_dict_model *dict = (struct cpe_dict_model *) oscap_iterator_next(dicts);
-		if (xccdf_policy_model_platforms_are_applicable_dict(model, dict, platforms)) {
-			oscap_iterator_free(dicts);
-			return true;
-		}
+		if (xccdf_policy_model_platforms_are_applicable_dict(model, dict, platforms))
+			ret = true;
 	}
 	oscap_iterator_free(dicts);
-	return false;
+
+	return ret;
 }
 
 bool xccdf_policy_model_item_is_applicable(struct xccdf_policy_model *model, struct xccdf_item *item)
@@ -1096,7 +1112,7 @@ _xccdf_policy_rule_evaluate(struct xccdf_policy * policy, const struct xccdf_rul
 		}
 
 		struct xccdf_check_import_iterator *check_import_it = xccdf_check_get_imports(check);
-		ret = xccdf_policy_evaluate_cb(policy, system_name, content_name, href, NULL, bindings, check_import_it);
+		ret = xccdf_policy_evaluate_cb(policy, system_name, content_name, href, bindings, check_import_it);
 		// the evaluation has filled check imports at this point, we can simply free the iterator
 		xccdf_check_import_iterator_free(check_import_it);
 
@@ -1674,6 +1690,8 @@ bool xccdf_policy_model_set_tailoring(struct xccdf_policy_model *model, struct x
 	xccdf_tailoring_free(model->tailoring);
 	model->tailoring = tailoring;
 
+	xccdf_tailoring_resolve(tailoring, xccdf_policy_model_get_benchmark(model));
+
 	return true;
 }
 
@@ -1751,35 +1769,28 @@ bool
 xccdf_policy_model_register_engine_and_query_callback(struct xccdf_policy_model *model, char *sys, xccdf_policy_engine_eval_fn eval_fn, void *usr, xccdf_policy_engine_query_fn query_fn)
 {
         __attribute__nonnull__(model);
-        callback * cb = oscap_alloc(sizeof(callback));
-        if (cb == NULL) return false;
-
-        cb->system   = sys;
-        cb->callback = eval_fn;
-        cb->usr      = usr;
-	cb->query_fn = query_fn;
-
-        return oscap_list_add(model->callbacks, cb);
+	struct xccdf_policy_engine *engine = xccdf_policy_engine_new(sys, eval_fn, usr, query_fn);
+	return oscap_list_add(model->engines, engine);
 }
 
-void xccdf_policy_model_unregister_callbacks(struct xccdf_policy_model *model, const char *sys)
+void xccdf_policy_model_unregister_engines(struct xccdf_policy_model *model, const char *sys)
 {
 	__attribute__nonnull__(model);
 	if (sys == NULL)
-		oscap_list_free(model->callbacks, (oscap_destruct_func) oscap_free);
+		oscap_list_free(model->engines, (oscap_destruct_func) oscap_free);
 	else {
 		struct oscap_list *rest = oscap_list_new();
-		struct oscap_iterator *cb_it = oscap_iterator_new(model->callbacks);
+		struct oscap_iterator *cb_it = oscap_iterator_new(model->engines);
 		while (oscap_iterator_has_more(cb_it)) {
-			callback *cb = oscap_iterator_next(cb_it);
-			if (oscap_streq(cb->system, sys))
-				oscap_free(cb);
+			struct xccdf_policy_engine *engine = oscap_iterator_next(cb_it);
+			if (xccdf_policy_engine_filter(engine, sys))
+				oscap_free(engine);
 			else
-				oscap_list_add(rest, cb);
+				oscap_list_add(rest, engine);
 		}
 		oscap_iterator_free(cb_it);
-		oscap_list_free0(model->callbacks);
-		model->callbacks = rest;
+		oscap_list_free0(model->engines);
+		model->engines = rest;
 	}
 }
 
@@ -1787,28 +1798,16 @@ bool xccdf_policy_model_register_start_callback(struct xccdf_policy_model * mode
 {
 
         __attribute__nonnull__(model);
-        callback_out * cb = oscap_alloc(sizeof(callback_out));
-        if (cb == NULL) return false;
-
-	cb->system = XCCDF_POLICY_OUTCB_START;
-        cb->callback = (void*)func;
-        cb->usr      = usr;
-
-        return oscap_list_add(model->callbacks, (callback *) cb);
+	struct reporter *reporter = reporter_new(XCCDF_POLICY_OUTCB_START, func, usr);
+        return oscap_list_add(model->callbacks, reporter);
 }
 
 bool xccdf_policy_model_register_output_callback(struct xccdf_policy_model * model, policy_reporter_output func, void * usr)
 {
 
         __attribute__nonnull__(model);
-        callback_out * cb = oscap_alloc(sizeof(callback_out));
-        if (cb == NULL) return false;
-
-	cb->system = XCCDF_POLICY_OUTCB_END;
-        cb->callback = (void*)func;
-        cb->usr      = usr;
-
-        return oscap_list_add(model->callbacks, (callback *) cb);
+	struct reporter *reporter = reporter_new(XCCDF_POLICY_OUTCB_END, func, usr);
+        return oscap_list_add(model->callbacks, reporter);
 }
 
 struct xccdf_result * xccdf_policy_get_result_by_id(struct xccdf_policy * policy, const char * id) {
@@ -1856,9 +1855,12 @@ struct xccdf_policy_model * xccdf_policy_model_new(struct xccdf_benchmark * benc
 	model->tailoring = NULL;
 	model->policies  = oscap_list_new();
         model->callbacks = oscap_list_new();
+	model->engines = oscap_list_new();
+
 	model->cpe_dicts = oscap_list_new();
 	model->cpe_lang_models = oscap_list_new();
 	model->cpe_oval_sessions = oscap_htable_new();
+	model->cpe_applicable_platforms = oscap_htable_new();
 
 	if (!xccdf_policy_model_add_default_cpe(model))
 	{
@@ -2163,9 +2165,9 @@ bool xccdf_policy_resolve(struct xccdf_policy * policy)
                 
             } else if (xccdf_item_get_type(item) == XCCDF_RULE) {
                 /* Perform all changes in rule */
-                if (xccdf_refine_rule_get_role(r_rule) > 0)
+                if ((int)xccdf_refine_rule_get_role(r_rule) > 0)
                     xccdf_rule_set_role((struct xccdf_rule *) item, xccdf_refine_rule_get_role(r_rule));
-                if (!xccdf_refine_rule_get_severity(r_rule) > 0)
+                if ((int)xccdf_refine_rule_get_severity(r_rule) > 0)
                     xccdf_rule_set_severity((struct xccdf_rule *) item, xccdf_refine_rule_get_severity(r_rule));
 
             } else {}/* TODO oscap_err ? */;
@@ -2254,6 +2256,13 @@ struct xccdf_result * xccdf_policy_evaluate(struct xccdf_policy * policy)
 	}
 	xccdf_item_iterator_free(item_it);
 
+	struct oscap_htable_iterator *it = oscap_htable_iterator_new(policy->model->cpe_applicable_platforms);
+	while (oscap_htable_iterator_has_more(it)) {
+		const char *key = oscap_htable_iterator_next_key(it);
+		xccdf_result_add_applicable_platform(result, key);
+	}
+	oscap_htable_iterator_free(it);
+
     xccdf_policy_add_result(policy, result);
 
 	xccdf_result_set_end_time_current(result);
@@ -2329,17 +2338,16 @@ const char *xccdf_policy_get_value_of_item(struct xccdf_policy * policy, struct
 	if (profile != NULL) {
 		/* Get set_value for this item */
 		struct xccdf_setvalue *s_value = NULL;
+		struct xccdf_setvalue *last_s_value = NULL;
 		struct xccdf_setvalue_iterator *s_value_it = xccdf_profile_get_setvalues(profile);
 		while (xccdf_setvalue_iterator_has_more(s_value_it)) {
 			s_value = xccdf_setvalue_iterator_next(s_value_it);
-			if (!strcmp(xccdf_setvalue_get_item(s_value), xccdf_value_get_id((struct xccdf_value *) item)))
-				break;
-			else
-				s_value = NULL;
+			if (strcmp(xccdf_setvalue_get_item(s_value), xccdf_value_get_id((struct xccdf_value *) item)) == 0)
+				last_s_value = s_value;
 		}
 		xccdf_setvalue_iterator_free(s_value_it);
-		if (s_value != NULL)
-			return xccdf_setvalue_get_value(s_value);
+		if (last_s_value != NULL)
+			return xccdf_setvalue_get_value(last_s_value);
 
 		/* We don't have set-value in profile, look for refine-value */
 		struct xccdf_refine_value_iterator *r_value_it = xccdf_profile_get_refine_values(profile);
@@ -2389,9 +2397,9 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
             if (r_rule == NULL) return item;
 
             new_item = (struct xccdf_item *) xccdf_rule_clone((struct xccdf_rule *) item);
-            if (xccdf_refine_rule_get_role(r_rule) > 0)
+            if ((int)xccdf_refine_rule_get_role(r_rule) > 0)
                 xccdf_rule_set_role((struct xccdf_rule *) new_item, xccdf_refine_rule_get_role(r_rule));
-            if (xccdf_refine_rule_get_severity(r_rule) > 0)
+            if ((int)xccdf_refine_rule_get_severity(r_rule) > 0)
                 xccdf_rule_set_severity((struct xccdf_rule *) new_item, xccdf_refine_rule_get_severity(r_rule));
             if (xccdf_refine_rule_weight_defined(r_rule))
                 xccdf_rule_set_weight((struct xccdf_rule *) new_item, xccdf_refine_rule_get_weight(r_rule));
@@ -2480,12 +2488,15 @@ static void _xccdf_policy_destroy_cpe_oval_session(void* ptr)
 void xccdf_policy_model_free(struct xccdf_policy_model * model) {
 
 	oscap_list_free(model->policies, (oscap_destruct_func) xccdf_policy_free);
-	xccdf_policy_model_unregister_callbacks(model, NULL);
+	xccdf_policy_model_unregister_engines(model, NULL);
+	oscap_list_free(model->callbacks, (oscap_destruct_func) oscap_free);
 	xccdf_tailoring_free(model->tailoring);
         xccdf_benchmark_free(model->benchmark);
+
 	oscap_list_free(model->cpe_dicts, (oscap_destruct_func) cpe_dict_model_free);
 	oscap_list_free(model->cpe_lang_models, (oscap_destruct_func) cpe_lang_model_free);
 	oscap_htable_free(model->cpe_oval_sessions, (oscap_destruct_func) _xccdf_policy_destroy_cpe_oval_session);
+	oscap_htable_free(model->cpe_applicable_platforms, NULL);
         oscap_free(model);
 }
 
