@@ -79,11 +79,18 @@ const char * oscap_path_to_schemas() {
 	return oscap_path_to("OSCAP_SCHEMA_PATH", OSCAP_SCHEMA_PATH);
 }
 
-
-const char * oscap_path_to_schematron() {
+static inline const char *oscap_path_to_xslt(void)
+{
 	return oscap_path_to("OSCAP_XSLT_PATH", OSCAP_XSLT_PATH);
 }
 
+OSCAP_DEPRECATED(
+const char * oscap_path_to_schematron() {
+	// It has never returned correct path to schematron files.
+	return oscap_path_to_xslt();
+}
+)
+
 const char * oscap_path_to_cpe() {
 	return oscap_path_to("OSCAP_CPE_PATH", OSCAP_CPE_PATH);
 }
@@ -508,7 +515,8 @@ struct oscap_schema_table_entry OSCAP_SCHEMATRON_TABLE[] = {
         {OSCAP_DOCUMENT_OVAL_DIRECTIVES,	"5.8",		"oval/5.8/oval-directives-schematron.xsl"},
         {OSCAP_DOCUMENT_OVAL_DIRECTIVES,	"5.9",		"oval/5.9/oval-directives-schematron.xsl"},
         {OSCAP_DOCUMENT_OVAL_DIRECTIVES,	"5.10",		"oval/5.10/oval-directives-schematron.xsl"},
-        {OSCAP_DOCUMENT_OVAL_DIRECTIVES,	"5.10.1",	"oval/5.10.1/oval-directives-schematron.xsl"}
+	{OSCAP_DOCUMENT_OVAL_DIRECTIVES,        "5.10.1",       "oval/5.10.1/oval-directives-schematron.xsl"},
+	{OSCAP_DOCUMENT_XCCDF,                  "1.2",          "xccdf/1.2/xccdf_1.2-schematron.xsl"},
 };
 
 int oscap_schematron_validate_document(const char *xmlfile, oscap_document_type_t doctype, const char *version, const char *outfile) {
@@ -552,7 +560,7 @@ int oscap_schematron_validate_document(const char *xmlfile, oscap_document_type_
 
 int oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params)
 {
-	return oscap_apply_xslt_path(xmlfile, xsltfile, outfile, params, oscap_path_to_schematron());
+	return oscap_apply_xslt_path(xmlfile, xsltfile, outfile, params, oscap_path_to_xslt());
 }
 
 int oscap_determine_document_type(const char *document, oscap_document_type_t *doc_type) {
@@ -566,6 +574,8 @@ int oscap_determine_document_type(const char *document, oscap_document_type_t *d
                 return -1;
         }
 
+	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, NULL);
+
         /* find root element */
         while (xmlTextReaderRead(reader) == 1
                && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
@@ -622,7 +632,7 @@ int oscap_determine_document_type(const char *document, oscap_document_type_t *d
                 return -1;
         }
 
-        dI("Identified document type: %s", elm_name);
+        dI("Identified document type: %s\n", elm_name);
 
         xmlFreeTextReader(reader);
         return 0;
