@@ -164,8 +164,11 @@ const char * oscap_path_to_schemas(void);
 
 /**
  * Function returns path used to locate OpenSCAP Schematron files
+ * @deprecated This function has been deprecated. It has never returned
+ * correct path to schematron files. This function may be dropped from
+ * the next version of the library.
  */
-const char * oscap_path_to_schematron(void);
+OSCAP_DEPRECATED(const char * oscap_path_to_schematron(void));
 
 /**
  * Function returns path used to locate OpenSCAP Default CPE files
