@@ -1,5 +1,5 @@
 /*
- * Copyright 2010,2011 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2010--2014 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -86,7 +86,10 @@ static struct oscap_module XCCDF_VALIDATE_XML = {
     .summary = "Validate XCCDF XML content",
     .usage = "xccdf-file.xml",
     .opt_parser = getopt_xccdf,
-    .func = app_xccdf_validate
+	.func = app_xccdf_validate,
+	.help = "Options:\n"
+		"   --schematron\r\t\t\t\t - Use schematron-based validation in addition to XML Schema\n"
+	,
 };
 
 static struct oscap_module XCCDF_VALIDATE = {
@@ -95,7 +98,10 @@ static struct oscap_module XCCDF_VALIDATE = {
     .summary = "Validate XCCDF XML content",
     .usage = "xccdf-file.xml",
     .opt_parser = getopt_xccdf,
-    .func = app_xccdf_validate
+	.func = app_xccdf_validate,
+	.help = "Options:\n"
+		"   --schematron\r\t\t\t\t - Use schematron-based validation in addition to XML Schema\n"
+	,
 };
 
 static struct oscap_module XCCDF_EXPORT_OVAL_VARIABLES = {
@@ -296,18 +302,14 @@ static int callback_scr_rule(struct xccdf_rule *rule, void *arg)
 	if (!selected)
 		return 0;
 
-	/* get the first title */
-	const char * title = NULL;
-	struct oscap_text_iterator * title_it = xccdf_rule_get_title(rule);
-	if (oscap_text_iterator_has_more(title_it))
-		title = oscap_text_get_text(oscap_text_iterator_next(title_it));
-	oscap_text_iterator_free(title_it);
+	const char *title = xccdf_policy_get_readable_item_title((struct xccdf_policy *)arg, (struct xccdf_item *) rule, NULL);
 
 	/* print */
 	if (isatty(1))
 		printf("Title\r\t\033[1m%s\033[0;0m\n", title);
 	else
 		printf("Title\r\t%s\n", title);
+	free((char *)title);
 	printf("Rule\r\t%s\n", rule_id);
 
 	struct xccdf_ident_iterator *idents = xccdf_rule_get_idents(rule);
@@ -681,7 +683,7 @@ int app_xccdf_resolve(const struct oscap_action *action)
 	else {
 		if (!xccdf_benchmark_resolve(bench))
 			fprintf(stderr, "Benchmark resolving failure (probably a dependency loop)!\n");
-
+		else
 		{
 			if (xccdf_benchmark_export(bench, action->f_results)) {
 				ret = OSCAP_OK;
@@ -938,6 +940,7 @@ bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 		{"remediate", no_argument, &action->remediate, 1},
 		{"hide-profile-info",	no_argument, &action->hide_profile_info, 1},
 		{"export-variables",	no_argument, &action->export_variables, 1},
+		{"schematron",          no_argument, &action->schematron, 1},
 	// end
 		{0, 0, 0, 0}
 	};
@@ -1035,6 +1038,15 @@ int app_xccdf_validate(const struct oscap_action *action) {
         else
                 result=OSCAP_OK;
 
+	if (action->schematron) {
+		ret = oscap_schematron_validate_document(action->f_xccdf, action->doctype, doc_version, NULL);
+		if (ret == -1) {
+			result = OSCAP_ERROR;
+		} else if (ret > 0) {
+			result = OSCAP_FAIL;
+		}
+	}
+
         if (result==OSCAP_FAIL)
 		validation_failed(action->f_xccdf, OSCAP_DOCUMENT_XCCDF, doc_version);
 
